from django.test import TestCase
from django.template import Context, Template


class CreateMetaTemplateTagTest(TestCase):
    def setUp(self):
        context = Context({})
        template_to_render = Template(
            '{% load pwa %}'
            '{% progressive_web_app_meta %}'
        )
        self.rendered_template = template_to_render.render(context)

    def test_has_tags(self):
        """Must contains the tags in HTML"""
        tags = [
            '<link rel="apple-touch-icon" href="/static/images/icons/icon-72x72.png" sizes="72x72">',
            '<link rel="apple-touch-icon" href="/static/images/icons/icon-96x96.png" sizes="96x96">',
            '<link rel="apple-touch-icon" href="/static/images/icons/icon-128x128.png" sizes="128x128">',
            '<link rel="apple-touch-icon" href="/static/images/icons/icon-144x144.png" sizes="144x144">',
            '<link rel="apple-touch-icon" href="/static/images/icons/icon-152x152.png" sizes="152x152">',
            '<link rel="apple-touch-icon" href="/static/images/icons/icon-192x192.png" sizes="192x192">',
            '<link rel="apple-touch-icon" href="/static/images/icons/icon-384x384.png" sizes="384x384">',
            '<link rel="apple-touch-icon" href="/static/images/icons/icon-512x512.png" sizes="512x512">',
            '<link rel="manifest" href="/manifest.json">',
            '<meta name="mobile-web-app-capable" content="yes">',
            '<meta name="theme-color" content="#000">',
            '<meta name="apple-mobile-web-app-capable" content="yes">',
            '<meta name="apple-mobile-web-app-title" content="MyApp">',
            '<meta name="application-name" content="MyApp">',
            '<meta name="apple-mobile-web-app-status-bar-style" content="default">',
            '<link rel="icon" sizes="512x512" href="/static/images/icons/icon-512x512.png">',
            '<link href="/static/images/icons/splash-640x1136.png" media="(device-width: 320px) and (device-height: 568px) and (-webkit-device-pixel-ratio: 2)" rel="apple-touch-startup-image"/>',
            '<link href="/static/images/icons/splash-750x1334.png" media="(device-width: 375px) and (device-height: 667px) and (-webkit-device-pixel-ratio: 2)" rel="apple-touch-startup-image"/>',
            '<link href="/static/images/icons/splash-1242x2208.png" media="(device-width: 621px) and (device-height: 1104px) and (-webkit-device-pixel-ratio: 3)" rel="apple-touch-startup-image"/>',
            '<link href="/static/images/icons/splash-1125x2436.png" media="(device-width: 375px) and (device-height: 812px) and (-webkit-device-pixel-ratio: 3)" rel="apple-touch-startup-image"/>',
            '<link href="/static/images/icons/splash-828x1792.png" media="(device-width: 414px) and (device-height: 896px) and (-webkit-device-pixel-ratio: 2)" rel="apple-touch-startup-image"/>',
            '<link href="/static/images/icons/splash-1242x2688.png" media="(device-width: 414px) and (device-height: 896px) and (-webkit-device-pixel-ratio: 3)" rel="apple-touch-startup-image"/>',
            '<link href="/static/images/icons/splash-1536x2048.png" media="(device-width: 768px) and (device-height: 1024px) and (-webkit-device-pixel-ratio: 2)" rel="apple-touch-startup-image"/>',
            '<link href="/static/images/icons/splash-1668x2224.png" media="(device-width: 834px) and (device-height: 1112px) and (-webkit-device-pixel-ratio: 2)" rel="apple-touch-startup-image"/>',
            '<link href="/static/images/icons/splash-1668x2388.png" media="(device-width: 834px) and (device-height: 1194px) and (-webkit-device-pixel-ratio: 2)" rel="apple-touch-startup-image"/>',
            '<link href="/static/images/icons/splash-2048x2732.png" media="(device-width: 1024px) and (device-height: 1366px) and (-webkit-device-pixel-ratio: 2)" rel="apple-touch-startup-image"/>',
            '<meta name="msapplication-TileColor" content="#fff">',
            '<meta name="msapplication-TileImage" content="/static/images/icons/icon-512x512.png">'
        ]
        for text in tags:
            with self.subTest():
                self.assertInHTML(text, self.rendered_template)

    def test_has_serviceworker(self):
        """Must have the script tag with serviceworker registration"""
        contents = [
            '<script type="text/javascript">',
            "if ('serviceWorker' in navigator) {",
            "navigator.serviceWorker.register('/serviceworker.js', {",
            "scope: '/'",
            "}).then(function (registration) {",
            "console.log('django-pwa: ServiceWorker registration successful with scope: ', registration.scope);",
            "}, function (err) {",
            "console.log('django-pwa: ServiceWorker registration failed: ', err);",
            "});",
            "</script>"
        ]

        for expected in contents:
            with self.subTest():
                self.assertIn(expected, self.rendered_template)



from django.test import TestCase
from django.shortcuts import resolve_url as r


class ServiceWorkerTest(TestCase):
    def setUp(self):
        self.response = self.client.get(r('serviceworker'))

    def test_get(self):
        """GET /serviceworker.js Should return status code 200"""
        self.assertEqual(200, self.response.status_code)


class ManifestTest(TestCase):
    def setUp(self):
        self.response = self.client.get(r('manifest'), format='json')

    def test_get(self):
        """GET /manifest.json Should return status code 200"""
        self.assertEqual(self.response.status_code, 200)

    def test_content_type_json(self):
        """The content type Must be JSON"""
        self.assertEqual(self.response['content-type'], 'application/json')

    def test_template(self):
        """Must have the template manifest.json"""
        self.assertTemplateUsed(self.response, 'manifest.json')

    def test_manifest_contains(self):
        """Must be the attributes to manitesf.json"""
        contents = [
            '"name":',
            '"short_name":',
            '"description":',
            '"start_url":',
            '"display":',
            '"scope":',
            '"background_color":',
            '"theme_color":',
            '"orientation":',
            '"icons":',
            '"dir":',
            '"lang":',
            '"status_bar":'
        ]
        for expected in contents:
            with self.subTest():
                self.assertContains(self.response, expected)


class OfflineTest(TestCase):
    def setUp(self):
        self.response = self.client.get(r('offline'))

    def test_get(self):
        """GET /offline Should return status code 200"""
        self.assertEqual(200, self.response.status_code)




from django.http import HttpResponse
from django.shortcuts import render

from . import app_settings


def service_worker(request):
    response = HttpResponse(open(app_settings.PWA_SERVICE_WORKER_PATH).read(), content_type='application/javascript')
    return response


def manifest(request):
    return render(request, 'manifest.json', {
        setting_name: getattr(app_settings, setting_name)
        for setting_name in dir(app_settings)
        if setting_name.startswith('PWA_')
    }, content_type='application/json')


def offline(request):
    return render(request, "offline.html")



from django.test import TestCase
from pwa import app_settings


class AppSettingsTest(TestCase):
    def test_has_defined(self):
        """Must have the attributes defined in app_settings.py"""
        attributes = [
            'PWA_SERVICE_WORKER_PATH',
            'PWA_APP_NAME',
            'PWA_APP_DESCRIPTION',
            'PWA_APP_ROOT_URL',
            'PWA_APP_THEME_COLOR',
            'PWA_APP_BACKGROUND_COLOR',
            'PWA_APP_SCOPE',
            'PWA_APP_DISPLAY',
            'PWA_APP_ORIENTATION',
            'PWA_APP_START_URL',
            'PWA_APP_FETCH_URL',
            'PWA_APP_ICONS',
            'PWA_APP_DIR',
            'PWA_APP_LANG',
            'PWA_APP_STATUS_BAR_COLOR'
        ]
        for attr in attributes:
            with self.subTest():
                self.assertTrue(hasattr(app_settings, attr))



import os
from setuptools import find_packages, setup

short_description = 'A Django app to include a manifest.json and Service Worker instance to enable progressive web ' \
                    'app behavior '

try:
    import pypandoc

    long_description = pypandoc.convert('README.md', 'rst')
except ModuleNotFoundError:
    long_description = short_description

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

install_requirements = [
    "django>=1.8",
]

setup(
    name='django-pwa',
    version='1.0.10',
    packages=find_packages(),
    install_requires=install_requirements,
    include_package_data=True,
    license='MIT License',
    description=short_description,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='http://github.com/silviolleite/django-pwa',
    author='Silvio Luis',
    author_email='silviolleite@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        'Framework :: Django :: 3.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)